searchData={"items":[{"type":"module","title":"JSV","doc":"This is the main API for the JSV library.","ref":"JSV.html"},{"type":"module","title":"Basic usage - JSV","doc":"The following snippet describes the general usage of the library in any context.\n\nThe rest of the documentation describes how to use JSV in the context of an application.\n\n```elixir\n# 1. Define a schema\nschema = %{\n  type: :object,\n  properties: %{\n    name: %{type: :string}\n  },\n  required: [:name]\n}\n\n# 2. Define a resolver\nresolver = {JSV.Resolver.BuiltIn, allowed_prefixes: [\"https://json-schema.org/\"]}\n\n# 3. Build the schema\nroot = JSV.build!(schema, resolver: resolver)\n\n# 4. Validate the data\ncase JSV.validate(%{\"name\" => \"Alice\"}, root) do\n  {:ok, data} ->\n    {:ok, data}\n\n  {:error, validation_error} ->\n    # Errors can be casted as JSON validator output to return them\n    # to the producer of the invalid data\n    {:error, JSON.encode!(JSV.normalize_error(validation_error))}\nend\n```","ref":"JSV.html#module-basic-usage"},{"type":"module","title":"Core concepts - JSV","doc":"","ref":"JSV.html#module-core-concepts"},{"type":"module","title":"Input schema format - JSV","doc":"\"Raw schemas\" are schemas defined in Elixir data structures such as `%{\"type\" =>\n\"integer\"}`.\n\nJSV does not accept JSON strings. You will need to decode the JSON strings\nbefore giving them to the build function. There are three different possible\nformats for a schema:\n\n1. **A boolean**. Booleans are valid schemas that accept anything (when `true`) or\n   reject everything (when `false`).\n2. **A map with binary keys and values** such as `%{\"type\" => \"integer\"}`.\n3. **A map with atom keys and values** such as `%{type :integer}`. The `:__struct__`\n   property of structs is safely ignored.\n\n   The `JSV.Schema` struct can be used for autocompletion, but it does not\n   provide any special behaviour over a raw map with atoms. The only difference\n   is that any `nil` value found in the struct will be ignored. `nil` values in\n   other maps or structs with atom keys are treated as-is (it's generally\n   invalid).\n\n   Atoms are converted to binaries internally so it is technically possible to\n   mix atom with binaries in map keys, but the behaviour for duplicate keys is\n   not defined: `%{\"type\" => \"string\", :type => \"integer\"}`.","ref":"JSV.html#module-input-schema-format"},{"type":"module","title":"Meta-schemas: Introduction to vocabularies - JSV","doc":"JSV was built in compliance with the vocabulary mechanism of JSON schema, to\nsupport custom and optional keywords in the schemas.\n\nIt may be more clear with an example:\n\n1. The well-known and official schema\n   `https://json-schema.org/draft/2020-12/schema` defines the following\n   vocabulary:\n\n   ```json\n   {\n     \"$vocabulary\": {\n       \"https://json-schema.org/draft/2020-12/vocab/core\": true,\n       \"https://json-schema.org/draft/2020-12/vocab/applicator\": true,\n       \"https://json-schema.org/draft/2020-12/vocab/unevaluated\": true,\n       \"https://json-schema.org/draft/2020-12/vocab/validation\": true,\n       \"https://json-schema.org/draft/2020-12/vocab/meta-data\": true,\n       \"https://json-schema.org/draft/2020-12/vocab/format-annotation\": true,\n       \"https://json-schema.org/draft/2020-12/vocab/content\": true\n     }\n   }\n   ```\n\n   The vocabulary is split in different parts, here one by object property.\n   More information can be found on the [official\n   website](https://json-schema.org/learn/glossary#vocabulary).\n\n2. Libraries such as JSV must map this vocabulary to implementations. For\n   instance, in JSV, the\n   `https://json-schema.org/draft/2020-12/json-schema-validation` part that\n   defines the `type` keyword is implemented with the\n   `JSV.Vocabulary.V202012.Validation` Elixir module.\n\n3. Finally, we can declare a schema that would like to use the `type` keyword.\n   To let the library know what implementation to use for that keyword, the\n   schema declares the `https://json-schema.org/draft/2020-12/schema` as its\n   meta-schema (using the `$schema` keyword).\n\n   ```json\n   {\n     \"$schema\": \"https://json-schema.org/draft/2020-12/schema\",\n     \"type\": \"integer\"\n   }\n   ```\n\n   This tells the library to pull the vocabulary from the meta-schema and apply\n   it to the schema.\n\n4. As JSV is compliant, it will use its implementation of\n   `https://json-schema.org/draft/2020-12/json-schema-validation` to validate\n   types.\n\n   This also means that you can use a custom meta schema to skip some parts of\n   the vocabulary, or add your own.","ref":"JSV.html#module-meta-schemas-introduction-to-vocabularies"},{"type":"module","title":"Resolvers overview - JSV","doc":"In order to build schemas properly, JSV needs to _resolve_ the schema as a first\nstep.\n\nResolving means fetching any remote resource whose data is needed and not\navailable ; basically resources whose\n[URIs](https://fr.wikipedia.org/wiki/Uniform_Resource_Identifier) are defined as\n`$schema`, `$ref` or `$dynamicRef`. Of course relative references do not need to\nbe fetched.\n\nThose URIs are generally URLs with the `http://` or `https://` scheme, but there\nare countless ways to fetch those URLs.\n\nFor security reasons, JSV does not provide a _default_ resolver that would fetch\nthose resources with an HTTP call. It is always required to provide the resolver\noption.\n\nFor convenience reasons, a _built-in_ resolver (`JSV.Resolver.BuiltIn`) is\nprovided but it still needs to be manually declared by users of the JSV library.\nThat resolver will download given URLs from the web. Refer to the documentation\nof this module for more information.","ref":"JSV.html#module-resolvers-overview"},{"type":"module","title":"Building schemas - JSV","doc":"In this chapter we will see how to build schemas from raw resources. The\nexamples will mention the `JSV.build/2` or `JSV.build!/2` functions\ninterchangeably. Everything described here applies to both.\n\nSchemas are built according to their meta-schema vocabulary. **JSV will assume\nthat the `$schema` value is `\"https://json-schema.org/draft/2020-12/schema\"` by\ndefault if not provided.**\n\nOnce built, a schema is converted into a `JSV.Root`, an internal representation\nof the schema that can be used to perform validation.\n\nIn a nutshell it boils down to the following:\n\n```elixir\nroot = JSV.build!(schema, resolver: resolver)\n```","ref":"JSV.html#module-building-schemas"},{"type":"module","title":"Built-in resolver - JSV","doc":"In order for that resolver to work, it must be able to decode JSON content from\nthe web. To do so, you will need to provide a valid JSON implementation:\n\n* From Elixir 1.18, the `JSON` module is automatically available in the standard\n  library.\n* JSV can use [Jason](https://hex.pm/packages/jason) if listed in your\n  dependencies with the  `\"~> 1.0\"` requirement.\n* JSV also supports [Poison](https://hex.pm/packages/poison) with the `\"~> 6.0\n  or ~> 5.0\"` requirement.","ref":"JSV.html#module-built-in-resolver"},{"type":"module","title":"Custom resolvers - JSV","doc":"The built-in resolver is here to help for common use cases, but you may need to\nresolve resources from custom locations instead of just fetching the web URL.\n\n`JSV.build/2` accepts any resolver implementation as long as it implements the\n`JSV.Resolver` behaviour. See the documentation of that behaviour for more\ninformation.\n\nIt is also possible to delegate to that resolver. If for instance we support a\n`my-company://` custom scheme, we could define the behaviour like so:\n\n```elixir\ndefmodule MyApp.SchemaResolver do\n  alias JSV.Resolver.BuiltIn\n\n  def resolve(\"my-company://\" <> _ = url, _opts) do\n    uri = URI.parse(url)\n    schema_dir = \"priv/schemas\"\n    # In this example the hostname in URL is ignored\n    schema_path = Path.join(schema_dir, uri.path)\n    json_schema = File.read!(schema_path)\n    JSON.decode(json_schema)\n  end\n\n  def resolve(\"https://\" <> _url, _opts) do\n    JSV.Resolver.BuiltIn.resolve(url,\n      allowed_prefixes: [\n        \"https://json-schema.org/\",\n        \"https://some-friend-company/\",\n        \"https://some-other-provider/\"\n      ]\n    )\n  end\nend\n```\n\nIt can be used as usual:\n\n```elixir\nJSV.build(raw_schema, resolver: MyApp.SchemaResolver)\n```","ref":"JSV.html#module-custom-resolvers"},{"type":"module","title":"Enable or disable format validation - JSV","doc":"By default, the `https://json-schema.org/draft/2020-12/schema` meta schema\n**does not perform format validation**. This is very counter intuitive, but it\nbasically means that the following code will return `{:ok, \"not a date\"}`:\n\n```elixir\nschema =\n  JSON.decode!(\"\"\"\n  {\n    \"type\": \"string\",\n    \"format\": \"date\"\n  }\n  \"\"\")\n\nroot = JSV.build!(schema, resolver: ...)\n\nJSV.validate(\"not a date\", root)\n```\n\nTo always enable format validation when building a root schema, provide the\n`formats: true` option to `JSV.build/2`:\n\n```elixir\nJSV.build(raw_schema, resolver: ..., formats: true)\n```\n\nThis is another reason to wrap `JSV.build` with a custom builder module!\n\nNote that format validation is determined at build time. There is no way to\nchange whether it is performed once the root schema is built.\n\n\nYou can also enable format validation by using the JSON Schema specification\nsemantics, though we strongly advise to just use the `:formats` option and call\nit a day.\n\nFor format validation to be enabled, a schema should declare the\n`https://json-schema.org/draft/2020-12/vocab/format-assertion` vocabulary\ninstead of the `https://json-schema.org/draft/2020-12/vocab/format-annotation`\none that is included by default in the\n`https://json-schema.org/draft/2020-12/schema` meta schema.\n\nSo, first we would declare a new meta schema:\n\n```json\n{\n    \"$id\": \"custom://with-formats-on/\",\n    \"$schema\": \"https://json-schema.org/draft/2020-12/schema\",\n    \"$vocabulary\": {\n        \"https://json-schema.org/draft/2020-12/vocab/core\": true,\n        \"https://json-schema.org/draft/2020-12/vocab/format-assertion\": true\n    },\n    \"$dynamicAnchor\": \"meta\",\n    \"allOf\": [\n        { \"$ref\": \"https://json-schema.org/draft/2020-12/meta/core\" },\n        { \"$ref\": \"https://json-schema.org/draft/2020-12/meta/format-assertion\" }\n    ]\n}\n```\n\nThis example is taken from the [JSON Schema Test\nSuite](https://github.com/json-schema-org/JSON-Schema-Test-Suite) codebase and\ndoes not includes all the vocabularies, only the assertion for the\nformats and the core vocabulary.\n\nThen we would declare our schema using that vocabulary to perform validation. Of\ncourse our resolver must be able to resolve the given URL for the new `$schema`\nproperty.\n\n```elixir\nschema =\n  JSON.decode!(\"\"\"\n  {\n    \"$schema\": \"custom://with-formats-on/\",\n    \"type\": \"string\",\n    \"format\": \"date\"\n  }\n  \"\"\")\n\nroot = JSV.build!(schema, resolver: ...)\n```\n\nWith this new meta-schema, `JSV.validate/2` would return an error tuple without\nneeding the `formats: true`.\n\n```elixir\n{:error, _} = JSV.validate(\"hello\", root)\n```\n\nIn this case, it is also possible to _disable_ the validation for schemas that\nuse a meta-schema where the assertion vocabulary is declared:\n\n```elixir\nJSV.build(raw_schema, resolver: ..., formats: false)\n```","ref":"JSV.html#module-enable-or-disable-format-validation"},{"type":"module","title":"Custom build modules - JSV","doc":"With that in mind, we suggest to define a custom module to wrap the\n`JSV.build/2` function, so the resolver, formats and vocabularies can be defined\nonly once.\n\nThat module could be implemented like this:\n\n```elixir\ndefmodule MyApp.SchemaBuilder do\n  def build_schema!(raw_schema) do\n    JSV.build!(raw_schema, resolver: MyApp.SchemaResolver, formats: true)\n  end\nend\n```","ref":"JSV.html#module-custom-build-modules"},{"type":"module","title":"Compile-time builds - JSV","doc":"It is strongly encouraged to build schemas at compile time, in order to avoid\nrepeating the build step for no good reason.\n\nFor instance, if we have this function that should validate external data:\n\n```elixir\n# Do not do this\n\ndef validate_order(order) do\n  root =\n    \"priv/schemas/order.schema.json\"\n    |> File.read!()\n    |> JSON.decode!()\n    |> MyApp.SchemaBuilder.build_schema!()\n\n  case JSV.validate(order, root) do\n    {:ok, _} -> OrderHandler.handle_order(order)\n    {:error, _} = err -> err\n  end\nend\n```\n\nThe schema will be built each time the function is called. Building a schema is\nactually pretty fast but it is a waste of resources nevertheless.\n\nOne could do the following to get a net performance gain:\n\n\n```elixir\n# Do this instead\n\n@order_schema \"priv/schemas/order.schema.json\"\n              |> File.read!()\n              |> JSON.decode!()\n              |> MyApp.SchemaBuilder.build_schema!()\n\ndefp order_schema, do: @order_schema\n\ndef validate_order(order) do\n  case JSV.validate(order, order_schema()) do\n    {:ok, _} -> OrderHandler.handle_order(order)\n    {:error, _} = err -> err\n  end\nend\n```\n\nYou can also define a module where all your schemas are built and exported as\nfunctions:\n\n```elixir\ndefmodule MyApp.Schemas do\n  schemas = [\n    order: \"tmp/order.schema.json\",\n    shipping: \"tmp/shipping.schema.json\"\n  ]\n\n  Enum.each(schemas, fn {fun, path} ->\n    root =\n      path\n      |> File.read!()\n      |> JSON.decode!()\n      |> MyApp.SchemaBuilder.build_schema!()\n\n    def unquote(fun)() do\n      unquote(Macro.escape(root))\n    end\n  end)\nend\n```\n\n...and use it elsewhere:\n\n```elixir\ndef validate_order(order) do\n  case JSV.validate(order, MyApp.Schemas.order()) do\n    {:ok, _} -> OrderHandler.handle_order(order)\n    {:error, _} = err -> err\n  end\nend\n```","ref":"JSV.html#module-compile-time-builds"},{"type":"module","title":"Validation - JSV","doc":"To validate a term, call the `JSV.validate/3` function like so:\n\n```elixir\nJSV.validate(data, root_schema, opts)\n```","ref":"JSV.html#module-validation"},{"type":"module","title":"General considerations - JSV","doc":"JSV supports all keywords of the 2020-12 specification except:\n\n* The return value of `JSV.validate/3` returns casted data. See the\n  documentation of that function for more information.\n* The `contentMediaType`, `contentEncoding` and `contentSchema` keywords. They\n  are ignored.  Future support for custom vocabularies will allow you to\n  validate data with such keywords.\n* The `format` keyword is largely supported but with many inconsistencies,\n  mostly due to differences between Elixir and JavaScript (JSON Schema is\n  largely based on JavaScript primitives). For most use cases, the differences\n  are negligible.\n* The `\"integer\"` type will transform floats into integer when the fractional\n  part is zero (such as `123.0`). Support for floating-point numbers with large\n  integer parts is using native Elixir semantics and may return incorrect\n  results:\n\n      iex(1)> trunc(123456789123456789123456789.0)\n      # returns:    123456789123456791337762816\n      #                             |\n      #                             | Difference starts here\n\n  This is because `123456789123456789123456789.0` is decoded as\n  `1.2345678912345679e26` from the shell as well as by the `JSON` module. This\n  value should have `25` zeroes when truncated but that will not be the case.\n\n  When dealing with such data it may be better to discard the casted data, or to\n  work with strings instead of floats.","ref":"JSV.html#module-general-considerations"},{"type":"module","title":"Formats - JSV","doc":"JSV supports multiple formats out of the box with its default implementation,\nbut some are only available under certain conditions that will be specified for\neach format.\n\nThe following listing describes the condition for support and return value type\nfor these default implementations. You can override those implementations by\nproviding your own, as well as providing new formats. This will be described\nlater in this document.\n\nAlso, note that by default, JSV format validation will return the original\nvalue, that is, the string form of the data. Some format validators can also\ncast the string to a more interesting data structure, for instance converting a\ndate string to a `Date` struct. You can enable returning casted values by\npassing the `cast_formats: true` option to `JSV.validate/3`.\n\nThe listing below describe values returned with that option enabled.\n\n**Important**: Some formats require the `abnf_parsec` library to be available.\nBut we have numerous problems with this library, yielding false negatives with\nmany inputs. An alternative solution will be implemented in future versions.\n\n<!-- block:formats-table -->\n#### date\n\n* **support**: Native.\n* **input**: `\"2020-04-22\"`\n* **output**: `~D[2020-04-22]`\n* The format is implemented with the native `Date` module.\n* The native `Date` module supports the `YYYY-MM-DD` format only. `2024`, `2024-W50`, `2024-12` will not be valid.\n\n#### date-time\n\n* **support**: Native.\n* **input**: `\"2025-01-02T00:11:23.416689Z\"`\n* **output**: `~U[2025-01-02 00:11:23.416689Z]`\n* The format is implemented with the native `DateTime` module.\n* The native `DateTime` module supports the `YYYY-MM-DD` format only for dates. `2024T...`, `2024-W50T...`, `2024-12T...` will not be valid.\n* Decimal precision is not capped to milliseconds. `2024-12-14T23:10:00.500000001Z` will be valid.\n\n#### duration\n\n* **support**: Requires Elixir 1.17\n* **input**: `\"P1DT4,5S\"`\n* **output**: `%Duration{day: 1, second: 4, microsecond: {500000, 1}}`\n* Elixir documentation states that _Only seconds may be specified with a decimal fraction, using either a comma or a full stop: P1DT4,5S_.\n* Elixir durations accept negative values.\n* Elixir durations accept out-of-range values, for instance more than 59 minutes.\n* Excessive precision (as in `\"PT10.0000000000001S\"`) will be valid.\n\n#### email\n\n* **support**: Requires `{:mail_address, \"~> 1.0\"}`.\n* **input**: `\"hello@json-schema.org\"`\n* **output**: `\"hello@json-schema.org\"` (same value)\n* Support is limited by the implementation of that library.\n* The `idn-email` format is not supported out-of-the-box.\n\n#### hostname\n\n* **support**: Native\n* **input**: `\"some-host\"`\n* **output**: `\"some-host\"` (same value)\n* Accepts numerical TLDs and single letter TLDs.\n\n#### ipv4\n\n* **support**: Native\n* **input**: `\"127.0.0.1\"`\n* **output**: `{127, 0, 0, 1}`\n\n#### ipv6\n\n* **support**: Native\n* **input**: `\"::1\"`\n* **output**: `{0, 0, 0, 0, 0, 0, 0, 1}`\n\n#### iri\n\n* **support**: Requires `{:abnf_parsec, \"~> 1.0\"}`.\n* **input**: `\"https://héhé.com/héhé\"`\n* **output**: `%URI{scheme: \"https\", authority: \"héhé.com\", userinfo: nil, host: \"héhé.com\", port: 443, path: \"/héhé\", query: nil, fragment: nil}`\n\n#### iri-reference\n\n* **support**: Requires `{:abnf_parsec, \"~> 1.0\"}`.\n* **input**: `\"//héhé\"`\n* **output**: `%URI{scheme: nil, authority: \"héhé\", userinfo: nil, host: \"héhé\", port: nil, path: nil, query: nil, fragment: nil}`\n\n#### json-pointer\n\n* **support**: Requires `{:abnf_parsec, \"~> 1.0\"}`.\n* **input**: `\"/foo/bar/baz\"`\n* **output**: `\"/foo/bar/baz\"` (same value)\n\n#### regex\n\n* **support**: Native\n* **input**: `\"[a-zA-Z0-9]\"`\n* **output**: `~r/[a-zA-Z0-9]/`\n* The format is implemented with the native `Regex` module.\n* The `Regex` module does not follow the `ECMA-262` specification.\n\n#### relative-json-pointer\n\n* **support**: Requires `{:abnf_parsec, \"~> 1.0\"}`.\n* **input**: `\"0/foo/bar\"`\n* **output**: `\"0/foo/bar\"` (same value)\n\n#### time\n\n* **support**: Native\n* **input**: `\"20:20:08.378586\"`\n* **output**: `~T[20:20:08.378586]`\n* The format is implemented with the native `Time` module.\n* The native `Time` implementation will completely discard the time offset information. Invalid offsets will be valid.\n* Decimal precision is not capped to milliseconds. `23:10:00.500000001` will be valid.\n\n#### unknown\n\n* **support**: Native\n* **input**: `\"anything\"`\n* **output**: `\"anything\"` (same value)\n* No validation or transformation is done.\n\n#### uri\n\n* **support**: Native, optionally uses `{:abnf_parsec, \"~> 1.0\"}`.\n* **input**: `\"http://example.com\"`\n* **output**: `%URI{scheme: \"http\", authority: \"example.com\", userinfo: nil, host: \"example.com\", port: 80, path: nil, query: nil, fragment: nil}`\n* Without the optional dependency, the `URI` module is used and a minimum checks on hostname and scheme presence are made.\n\n#### uri-reference\n\n* **support**: Native, optionally uses `{:abnf_parsec, \"~> 1.0\"}`.\n* **input**: `\"/example-path\"`\n* **output**: `%URI{scheme: nil, userinfo: nil, host: nil, port: nil, path: \"/example-path\", query: nil, fragment: nil}`\n* Without the optional dependency, the `URI` module will cast most non url-like strings as a `path`.\n\n#### uri-template\n\n* **support**: Requires `{:abnf_parsec, \"~> 1.0\"}`.\n* **input**: `\"http://example.com/search{?query,lang}\"`\n* **output**: `\"http://example.com/search{?query,lang}\"` (same value)\n\n#### uuid\n\n* **support**: Native\n* **input**: `\"bf22824c-c8a4-11ef-9642-0fdaf117eeb9\"`\n* **output**: `\"bf22824c-c8a4-11ef-9642-0fdaf117eeb9\"` (same value)\n\n\n<!-- endblock:formats-table -->","ref":"JSV.html#module-formats"},{"type":"module","title":"Custom formats - JSV","doc":"In order to provide custom formats, or to override default implementations for\nformats, you may provide a list of modules as the value for the `:formats`\noptions of `JSV.build/2`. Such modules must implement the `JSV.FormatValidator`\nbehaviour.\n\nFor instance:\n\n```elixir\ndefmodule CustomFormats do\n  @behaviour JSV.FormatValidator\n\n  @impl true\n  def supported_formats do\n    [\"greeting\"]\n  end\n\n  @impl true\n  def validate_cast(\"greeting\", data) do\n    case data do\n      \"hello \" <> name -> {:ok, %Greeting{name: name}}\n      _ -> {:error, :invalid_greeting}\n    end\n  end\nend\n```\n\nWith this module you can now call the builder with it:\n\n```elixir\nJSV.build!(raw_schema, resolver: ..., formats: [CustomFormats])\n```\n\nNote that this will disable all other formats. If you need to still support the\ndefault formats, a helper is available:\n\n```elixir\nJSV.build!(raw_schema,\n  resolver: ...,\n  formats: [CustomFormats | JSV.default_format_validator_modules()]\n)\n```\n\nFormat validation modules are checked during the build phase, in order. So you\ncan override any format defined by a module that comes later in the list,\nincluding the default modules.","ref":"JSV.html#module-custom-formats"},{"type":"module","title":"Development - JSV","doc":"","ref":"JSV.html#module-development"},{"type":"module","title":"Contributing - JSV","doc":"Pull requests are welcome given appropriate tests and documentation.","ref":"JSV.html#module-contributing"},{"type":"module","title":"Roadmap - JSV","doc":"- Support for custom vocabularies\n- Declare a JSON codec module directly as built-in resolver option. This will be\n  implemented if needed, we do not think there will be a strong demand for that.","ref":"JSV.html#module-roadmap"},{"type":"function","title":"JSV.build/2","doc":"Builds the schema as a `JSV.Root` schema for validation.","ref":"JSV.html#build/2"},{"type":"function","title":"Options - JSV.build/2","doc":"* `:resolver` - Required. The `JSV.Resolver` behaviour implementation module to\n  retrieve schemas identified by an URL.\n\n  Accepts a `module` or a `{module, options}` tuple.\n  The options can be any term and will be given to the\n  `resolve/2` callback of the module.\n\n* `:default_meta` (`t:String.t/0`) - The meta schema to use for resolved schemas that do not define a `\"$schema\"` property. The default value is `\"https://json-schema.org/draft/2020-12/schema\"`.\n\n* `:formats` - Controls the validation of strings with the `\"format\"` keyword.\n\n  * `nil` - Formats are validated according to the meta-schema vocabulary.\n  * `true` - Enforces validation with the built-in validator modules.\n  * `false` - Disables all format validation.\n  * `[Module1, Module2,...]` – set those modules as validators. Disables the built-in format validator modules.\n     The default validators can be included manually in the list, see `default_format_validator_modules/0`.\n\n  The default value is `nil`.","ref":"JSV.html#build/2-options"},{"type":"function","title":"JSV.build!/2","doc":"Same as `build/2` but raises on error.","ref":"JSV.html#build!/2"},{"type":"function","title":"JSV.default_format_validator_modules/0","doc":"Returns the list of format validator modules that are used when a schema is\nbuilt with format validation enabled and the `:formats` option to `build/2` is\n`true`.","ref":"JSV.html#default_format_validator_modules/0"},{"type":"function","title":"JSV.default_meta/0","doc":"Returns the default meta schema used when the `:default_meta` option is not\nset in `build/2`.\n\nCurrently returns \"https://json-schema.org/draft/2020-12/schema\".","ref":"JSV.html#default_meta/0"},{"type":"function","title":"JSV.normalize_error/1","doc":"","ref":"JSV.html#normalize_error/1"},{"type":"function","title":"JSV.validate/3","doc":"Validate the data with the given schema. The schema must be a `JSV.Root`\nstruct generated with `build/2`.\n\n**Important**: this function returns casted data:\n\n* If the `:cast_formats` option is enabled, string values may be transformed\n  in other data structures. Refer to the \"Formats\" section of the `JSV`\n  documentation for more information.\n* The JSON Schema specification states that `123.0` is a valid integer. This\n  function will return `123` instead. This may return invalid data for floats\n  with very large integer parts. As always when dealing with JSON and floats,\n  use strings.\n* Future versions of the library will allow to cast raw data into Elixir\n  structs.","ref":"JSV.html#validate/3"},{"type":"function","title":"Options - JSV.validate/3","doc":"* `:cast_formats` (`t:boolean/0`) - When enabled format validators will return casted values, for instance a `Date` struct instead of the date as string. It has no effect when the schema was not built with formats enabled. The default value is `false`.","ref":"JSV.html#validate/3-options"},{"type":"module","title":"JSV.Schema","doc":"This module defines a struct where all the supported keywords of the JSON\nschema specification are defined as keys. Text editors that can predict the\nstruct keys will make autocompletion available when writing schemas.","ref":"JSV.Schema.html"},{"type":"module","title":"Using in build - JSV.Schema","doc":"The `JSV.Schema` struct can be given to `JSV.build/2`:\n\n    schema = %JSV.Schema{type: :integer}\n    JSV.build(schema, options())\n\nBecause Elixir structs always contain all their defined keys, writing a schema\nas `%JSV.Schema{type: :integer}` is actually defining the following:\n\n    %JSV.Schema{\n      type: :integer,\n      \"$id\": nil\n      additionalItems: nil,\n      additionalProperties: nil,\n      allOf: nil,\n      anyOf: nil,\n      contains: nil,\n      # etc...\n    }\n\nFor that reason, when giving a `JSV.Schema` struct to\n`JSV.build/2`, any `nil` value is ignored. This is not the case with other\nstrucs or maps.\n\nNote that `JSV.build/2` does not require `JSV.Schema` structs, any\nmap with binary or atom keys is accepted.\n\nThis is also why the `JSV.Schema` struct does not define the\n`const` keyword, because `nil` is a valid value for that keyword but there is\nno way to know if the value was omitted or explicitly defined as `nil`. To\ncircumvent that you may use the `enum` keyword or just use a regular map\ninstead of this module's struct:\n\n    %JSV.Schema{enum: [nil]}\n    # OR\n    %{const: nil}","ref":"JSV.Schema.html#module-using-in-build"},{"type":"module","title":"Functional helpers - JSV.Schema","doc":"This module also exports a small range of utility functions to ease writing\nschemas in a functional way.\n\nThis is mostly useful when generating schemas dynamically, or for shorthands.\n\nFor instance, instead of writing the following:\n\n    %Schema{\n      type: :object,\n      properties: %{\n        name: %Schema{type: :string, description: \"the name of the user\"},\n        age: %Schema{type: :integer, description: \"the age of the user\"}\n      },\n      required: [:name, :age]\n    }\n\nOne can write:\n\n    %Schema{}\n    |> Schema.props(\n      name: Schema.string(description: \"the name of the user\"),\n      age: Schema.integer(description: \"the age of the user\")\n    )\n    |> Schema.required([:name, :age])","ref":"JSV.Schema.html#module-functional-helpers"},{"type":"type","title":"JSV.Schema.base/0","doc":"","ref":"JSV.Schema.html#t:base/0"},{"type":"function","title":"JSV.Schema.boolean/1","doc":"Returns a schema with `type: :boolean`.","ref":"JSV.Schema.html#boolean/1"},{"type":"function","title":"JSV.Schema.integer/1","doc":"Returns a schema with `type: :integer`.","ref":"JSV.Schema.html#integer/1"},{"type":"function","title":"JSV.Schema.items/2","doc":"Returns a schema with `type: :array` and `items: item_schema`.","ref":"JSV.Schema.html#items/2"},{"type":"function","title":"JSV.Schema.new/1","doc":"Returns a new `JSV.Schema` struct with the given key/values.","ref":"JSV.Schema.html#new/1"},{"type":"function","title":"JSV.Schema.number/1","doc":"Returns a schema with `type: :number`.","ref":"JSV.Schema.html#number/1"},{"type":"function","title":"JSV.Schema.object/1","doc":"Returns a schema with `type: :object`.\n\nSee `props/2` to define the properties as well.","ref":"JSV.Schema.html#object/1"},{"type":"function","title":"JSV.Schema.override/2","doc":"Updates the given `JSV.Schema` struct with the given key/values.\n\nAccepts `nil` as the base schema, which is equivalent to `new(overrides)`.","ref":"JSV.Schema.html#override/2"},{"type":"function","title":"JSV.Schema.props/2","doc":"Returns a schema with the `type: :object` and the given `properties`.","ref":"JSV.Schema.html#props/2"},{"type":"type","title":"JSV.Schema.prototype/0","doc":"","ref":"JSV.Schema.html#t:prototype/0"},{"type":"function","title":"JSV.Schema.ref/2","doc":"Returns a schema with `\"$ref\": ref`.","ref":"JSV.Schema.html#ref/2"},{"type":"function","title":"JSV.Schema.required/2","doc":"Adds the given key or keys in the base schema `:required` property. Previous\nvalues are preserved.","ref":"JSV.Schema.html#required/2"},{"type":"function","title":"JSV.Schema.string/1","doc":"Returns a schema with `type: :string`.","ref":"JSV.Schema.html#string/1"},{"type":"type","title":"JSV.Schema.t/0","doc":"","ref":"JSV.Schema.html#t:t/0"},{"type":"exception","title":"JSV.BuildError","doc":"A simple wrapper for errors returned from `JSV.build/2`.","ref":"JSV.BuildError.html"},{"type":"function","title":"JSV.BuildError.of/1","doc":"Wraps the given term as the `reason` in a `JSV.BuildError` struct.","ref":"JSV.BuildError.html#of/1"},{"type":"module","title":"JSV.Builder","doc":"Internal logic to build raw schemas into `JSV.Root` structs.","ref":"JSV.Builder.html"},{"type":"function","title":"JSV.Builder.build/2","doc":"Builds the given raw schema into a `JSV.Root` struct.","ref":"JSV.Builder.html#build/2"},{"type":"function","title":"JSV.Builder.build_sub/2","doc":"","ref":"JSV.Builder.html#build_sub/2"},{"type":"type","title":"JSV.Builder.buildable/0","doc":"","ref":"JSV.Builder.html#t:buildable/0"},{"type":"function","title":"JSV.Builder.ensure_resolved/2","doc":"Ensures that the remote resource that the given reference or key points to is\nfetched in the builder internal cache","ref":"JSV.Builder.html#ensure_resolved/2"},{"type":"function","title":"JSV.Builder.fetch_resolved/2","doc":"Returns the raw schema identified by the given key. Use `ensure_resolved/2`\nbefore if the resource may not have been fetched.","ref":"JSV.Builder.html#fetch_resolved/2"},{"type":"function","title":"JSV.Builder.new/1","doc":"Returns a new builder. Builders are not reusable ; a fresh builder must be\nmade for each different root schema.","ref":"JSV.Builder.html#new/1"},{"type":"type","title":"JSV.Builder.raw_schema/0","doc":"","ref":"JSV.Builder.html#t:raw_schema/0"},{"type":"type","title":"JSV.Builder.resolvable/0","doc":"","ref":"JSV.Builder.html#t:resolvable/0"},{"type":"function","title":"JSV.Builder.stage_build/2","doc":"Adds a new key to be built later. A key is generatlly derived from a\nreference.","ref":"JSV.Builder.html#stage_build/2"},{"type":"type","title":"JSV.Builder.t/0","doc":"","ref":"JSV.Builder.html#t:t/0"},{"type":"function","title":"JSV.Builder.vocabulary_enabled?/2","doc":"","ref":"JSV.Builder.html#vocabulary_enabled?/2"},{"type":"module","title":"JSV.Codec","doc":"JSON encoder/decoder based on available implementation.","ref":"JSV.Codec.html"},{"type":"function","title":"JSV.Codec.decode/1","doc":"Equivalent to `JSON.decode/1`.","ref":"JSV.Codec.html#decode/1"},{"type":"function","title":"JSV.Codec.decode!/1","doc":"Equivalent to `JSON.decode!/1`.","ref":"JSV.Codec.html#decode!/1"},{"type":"function","title":"JSV.Codec.encode!/1","doc":"Equivalent to `JSON.encode!/1`.","ref":"JSV.Codec.html#encode!/1"},{"type":"module","title":"JSV.Key","doc":"Helpers around the different key formats used in the resolver, builder and\nvalidator states to index sub schemas, referenced schemas, anchor or meta\nschemas.\n\nFor instance:\n\n* We have a `{\"$ref\": \"http://some-schema/#$defs/order\"}` JSON schema.\n* The builder will build the `$ref` keyword as a key: `{:pointer,\n  \"http://some-schema/\", [\"$defs\",\"order\"]}`.\n* The builder, via the resolver, will fetch `http://some-schema/`, store it\n  locally and build validators. Those validators will be stored under the same\n  key (`{:pointer, \"http://some-schema/\", [\"$defs\",\"order\"]}`) in the root\n  schema.\n* When the validator will validate the reference, it will fetch that key from\n  the root schema and apply the retrieved validators to the data.","ref":"JSV.Key.html"},{"type":"type","title":"JSV.Key.anchor/0","doc":"","ref":"JSV.Key.html#t:anchor/0"},{"type":"type","title":"JSV.Key.dynamic_anchor/0","doc":"","ref":"JSV.Key.html#t:dynamic_anchor/0"},{"type":"function","title":"JSV.Key.for_anchor/2","doc":"Returns an anchor type key.","ref":"JSV.Key.html#for_anchor/2"},{"type":"function","title":"JSV.Key.for_dynamic_anchor/2","doc":"Returns a dynamic anchor type key.","ref":"JSV.Key.html#for_dynamic_anchor/2"},{"type":"function","title":"JSV.Key.for_pointer/2","doc":"Returns a pointer type key.","ref":"JSV.Key.html#for_pointer/2"},{"type":"function","title":"JSV.Key.namespace_of/1","doc":"Returns the namespace of the key.","ref":"JSV.Key.html#namespace_of/1"},{"type":"type","title":"JSV.Key.ns/0","doc":"","ref":"JSV.Key.html#t:ns/0"},{"type":"function","title":"JSV.Key.of/1","doc":"Creates a new key from an external or local reference.","ref":"JSV.Key.html#of/1"},{"type":"type","title":"JSV.Key.pointer/0","doc":"","ref":"JSV.Key.html#t:pointer/0"},{"type":"type","title":"JSV.Key.t/0","doc":"","ref":"JSV.Key.html#t:t/0"},{"type":"function","title":"JSV.Key.to_iodata/1","doc":"Returns a string representation of the key, in a URL/JSON pointer format,\nas chardata.","ref":"JSV.Key.html#to_iodata/1"},{"type":"module","title":"JSV.RNS","doc":"A \"namespace\" for a schema ID or reference. In the JSV library, a namespace\nfor a schema represents the document the schema belongs to. When it is an URL,\nit is the scheme, host and path, ignoring the query string and the fragment.\n\nThat is basically a URI but with extra support for URNs (`urn:isbn:1234` is\nrepresented as `urn://isbn/1234`).","ref":"JSV.RNS.html"},{"type":"function","title":"JSV.RNS.derive/2","doc":"Returns a new string namespace by appending a relative child path to a parent\nnamespace. If the child is absolute or `:root`, returns the child.","ref":"JSV.RNS.html#derive/2"},{"type":"function","title":"JSV.RNS.parse/1","doc":"Parses the given URL or URN and returns an internal representation of its\nnamespace.\n\nAlso accepts `:root` for root schemas without `$id`.","ref":"JSV.RNS.html#parse/1"},{"type":"type","title":"JSV.RNS.t/0","doc":"","ref":"JSV.RNS.html#t:t/0"},{"type":"function","title":"JSV.RNS.to_ns/1","doc":"Returns the string value of the namespace, or `:root`.","ref":"JSV.RNS.html#to_ns/1"},{"type":"module","title":"JSV.Ref","doc":"Representation of a JSON Schema reference (`$ref` or `$dynamicRef`).","ref":"JSV.Ref.html"},{"type":"function","title":"JSV.Ref.escape_json_pointer/1","doc":"Encodes the given string as a JSON representation of a JSON pointer, that is\nwith `~` as `~0` and `/` as `~1`.","ref":"JSV.Ref.html#escape_json_pointer/1"},{"type":"type","title":"JSV.Ref.ns/0","doc":"","ref":"JSV.Ref.html#t:ns/0"},{"type":"function","title":"JSV.Ref.parse/2","doc":"Creates a new reference from an URL, relative to the given namespace.\n\nIf the URL is absolute and its namespace is different from the given\nnamespace, returns an absolute URL.","ref":"JSV.Ref.html#parse/2"},{"type":"function","title":"JSV.Ref.parse_dynamic/2","doc":"Like `parse/2` but flags the reference as dynamic.","ref":"JSV.Ref.html#parse_dynamic/2"},{"type":"type","title":"JSV.Ref.t/0","doc":"","ref":"JSV.Ref.html#t:t/0"},{"type":"behaviour","title":"JSV.Resolver","doc":"Resolves remote resources when building a JSON schema.\n\nA resolver implementation is needed for `JSV.build/2`.\n\nThere are no specific rules on how to build a proper resolver. Anything is\nvalid as long as it returns a JSON schema (or an error) for a given URI.\n\nCustom resolvers are most often used for:\n\n- Ability to resolve URLs such as `my-company://some-id/` where the\n  implementation knows a directory to retrieve that path from.\n- Ability to resolve `https://` URLs with custom network setups involving\n  authentication, proxies, etc.\n- Returning hardcoded schemas directly from the codebase, or files from the\n  codebase.\n- Returning a different schema depending on the environment, whether this is a\n  good idea or not.","ref":"JSV.Resolver.html"},{"type":"function","title":"JSV.Resolver.fetch_meta/2","doc":"Returns the raw schema identified by the given namespace if was previously\nresolved as a meta-schema.","ref":"JSV.Resolver.html#fetch_meta/2"},{"type":"function","title":"JSV.Resolver.fetch_resolved/2","doc":"Returns the raw schema identified by the given key if was previously resolved.","ref":"JSV.Resolver.html#fetch_resolved/2"},{"type":"function","title":"JSV.Resolver.new/2","doc":"Returns a new resolver, with the given `module` for behaviour implementation,\nand a default meta-schema URL to use with schemas that do not declare a\n`$schema` property.","ref":"JSV.Resolver.html#new/2"},{"type":"type","title":"JSV.Resolver.raw_schema/0","doc":"","ref":"JSV.Resolver.html#t:raw_schema/0"},{"type":"type","title":"JSV.Resolver.resolvable/0","doc":"","ref":"JSV.Resolver.html#t:resolvable/0"},{"type":"function","title":"JSV.Resolver.resolve/2","doc":"Returns the resolver with the remote resolvable resource fetched in the\ninternal cache of the resolver.","ref":"JSV.Resolver.html#resolve/2"},{"type":"callback","title":"JSV.Resolver.resolve/2","doc":"Receives an URI and the options passed in the resolver tuple to `JSV.build/2`\nand returns a result tuple for a raw JSON schema, that is a map with binary\nkeys or a boolean.","ref":"JSV.Resolver.html#c:resolve/2"},{"type":"function","title":"JSV.Resolver.resolve_root/2","doc":"Adds the given raw schema as a pre-resolved schema, using the `:root`\nnamespace if the schema does not contain a `$id` property.","ref":"JSV.Resolver.html#resolve_root/2"},{"type":"opaque","title":"JSV.Resolver.t/0","doc":"","ref":"JSV.Resolver.html#t:t/0"},{"type":"module","title":"JSV.Resolver.BuiltIn","doc":"This is the built-in resolver provided to simplify building schemas. This is\nan HTTP resolver that will attempt resolve resources with an HTTP GET call to\nthe given URI.\n\nTo use this resolver, you must provide a list of allowed prefixes. The\nresolver will not attempt to fetch URLs that do not start with one of those\nprefixes.\n\nThe resolver uses two caches: a persistent cache on the disk that lives\nthrough multiple runtimes and compilations, and a memory cache that is only\nused within the same runtime, _i.e._ one compilation or one execution of your\napplication.\n\nNote that there is no time-to-live at the moment, cache is _forever_ in both\ncases: until the files are deleted in the case of the disk cache, or until the\nruntime is shut down for the memory cache.","ref":"JSV.Resolver.BuiltIn.html"},{"type":"module","title":"Options - JSV.Resolver.BuiltIn","doc":"This resolver supports the following options:\n\n- `:allowed_prefixes` - This option is mandatory and contains the allowed\n  prefixes to download from.\n- `:cache_dir` - The path of a directory to cache downloaded resources. The\n  default value can be retrieved with `default_cache_dir/0` and is based on\n  `System.tmp_dir!/0`. The option also accepts `false` to disable that cache.","ref":"JSV.Resolver.BuiltIn.html#module-options"},{"type":"module","title":"Example - JSV.Resolver.BuiltIn","doc":"resolver_opts = [allowed_prefixes: [\"https://json-schema.org/\"], cache_dir: \"_build/custom/dir\"]\n    JSV.build(schema, resolver: {JSV.Resolver.BuiltIn, resolver_opts})","ref":"JSV.Resolver.BuiltIn.html#module-example"},{"type":"function","title":"JSV.Resolver.BuiltIn.default_cache_dir/0","doc":"Returns the default directory used by the disk-based cache.","ref":"JSV.Resolver.BuiltIn.html#default_cache_dir/0"},{"type":"module","title":"JSV.Resolver.Cache","doc":"An in-memory cache to store resources resolved by the `JSV.Resolver.BuiltIn`\nresolver implementation.\n\nThe cache implementation is very basic an serves as a dependency-free solution\nfor applications building schemas at runtime.\n\nIf all your schemas are built at compile-time, you do not need this cache\nbecause it is not started during compilation, for simplicity sake.\n\nA cache instance is automatically started with the `:jsv` OTP application with\n`Cache` as a name. You can use the cache by passing that name as the first\narguments of the functions, for instance:\n\n    Cache.get_or_generate(Cache, :some_key, fn -> {:ok, \"value\"} end)","ref":"JSV.Resolver.Cache.html"},{"type":"function","title":"JSV.Resolver.Cache.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"JSV.Resolver.Cache.html#child_spec/1"},{"type":"function","title":"JSV.Resolver.Cache.get_or_generate/3","doc":"","ref":"JSV.Resolver.Cache.html#get_or_generate/3"},{"type":"function","title":"JSV.Resolver.Cache.start_link/1","doc":"Starts a cache identified with a name.","ref":"JSV.Resolver.Cache.html#start_link/1"},{"type":"function","title":"Options - JSV.Resolver.Cache.start_link/1","doc":"* `:name` (`t:atom/0`) - Required. The name for the cache and the public ETS table.\n\n\n\nThis function also supports other `GenServer` options.","ref":"JSV.Resolver.Cache.html#start_link/1-options"},{"type":"module","title":"JSV.Resolver.Resolved","doc":"Represents a cache entry for a resolved resource.","ref":"JSV.Resolver.Resolved.html"},{"type":"type","title":"JSV.Resolver.Resolved.t/0","doc":"","ref":"JSV.Resolver.Resolved.html#t:t/0"},{"type":"module","title":"JSV.BooleanSchema","doc":"Represents a boolean schema. Boolean schemas accept or reject any data\naccording to their boolean value.\n\nThis is very often used with the `additionalProperties` keyword.","ref":"JSV.BooleanSchema.html"},{"type":"function","title":"JSV.BooleanSchema.of/1","doc":"Returns a `JSV.BooleanSchema` struct wrapping the given boolean.","ref":"JSV.BooleanSchema.html#of/1"},{"type":"type","title":"JSV.BooleanSchema.t/0","doc":"","ref":"JSV.BooleanSchema.html#t:t/0"},{"type":"module","title":"JSV.ErrorFormatter","doc":"Error formatting loosely following the guidelines described at\nhttps://json-schema.org/blog/posts/fixing-json-schema-output\n\nErrors are grouped by similar instance location (the bit of data that was\ninvalidated) and schema location (the part of the schema that invalidated it).","ref":"JSV.ErrorFormatter.html"},{"type":"type","title":"JSV.ErrorFormatter.annotation/0","doc":"","ref":"JSV.ErrorFormatter.html#t:annotation/0"},{"type":"type","title":"JSV.ErrorFormatter.collected_error/0","doc":"","ref":"JSV.ErrorFormatter.html#t:collected_error/0"},{"type":"function","title":"JSV.ErrorFormatter.format_data_path/1","doc":"","ref":"JSV.ErrorFormatter.html#format_data_path/1"},{"type":"function","title":"JSV.ErrorFormatter.normalize_error/1","doc":"Returns a JSON-able version of the errors contained in the ValidationError.\n\nThis is generatlly useful to generate HTTP API responses or message broker\nresponses.","ref":"JSV.ErrorFormatter.html#normalize_error/1"},{"type":"type","title":"JSV.ErrorFormatter.raw_path/0","doc":"","ref":"JSV.ErrorFormatter.html#t:raw_path/0"},{"type":"function","title":"JSV.ErrorFormatter.valid_unit/1","doc":"Returns an output unit with `valid: true` for the given\n`JSV.Validator`. This can be substitued to an Error struct in the\nnested details of an error. Mostly used to show multiple validated schemas\nwith `:oneOf`.","ref":"JSV.ErrorFormatter.html#valid_unit/1"},{"type":"module","title":"JSV.Root","doc":"Internal representation of a JSON schema built with `JSV.build/2`.\n\nThe original schema, in its string-keys form, can be retrieved in the `:raw`\nkey of the struct.","ref":"JSV.Root.html"},{"type":"type","title":"JSV.Root.t/0","doc":"","ref":"JSV.Root.html#t:t/0"},{"type":"module","title":"JSV.Subschema","doc":"Build result for a raw map schema.","ref":"JSV.Subschema.html"},{"type":"type","title":"JSV.Subschema.t/0","doc":"","ref":"JSV.Subschema.html#t:t/0"},{"type":"type","title":"JSV.Subschema.validators/0","doc":"","ref":"JSV.Subschema.html#t:validators/0"},{"type":"exception","title":"JSV.ValidationError","doc":"The data structure returned by `JSV.validate/3` when validation fails.","ref":"JSV.ValidationError.html"},{"type":"function","title":"JSV.ValidationError.of/1","doc":"Wraps the given `JSV.Validator.Error` list into an `JSV.ValidationError`\nexception.","ref":"JSV.ValidationError.html#of/1"},{"type":"type","title":"JSV.ValidationError.t/0","doc":"","ref":"JSV.ValidationError.html#t:t/0"},{"type":"module","title":"JSV.Validator","doc":"This is the home of the recursive validation logic.\n\nThe validator is called on the root schema, and may be called by vocabulary\nimplementations to validate sub parts of the data built withing each\nvocabulary module.","ref":"JSV.Validator.html"},{"type":"type","title":"JSV.Validator.context/0","doc":"","ref":"JSV.Validator.html#t:context/0"},{"type":"function","title":"JSV.Validator.context/3","doc":"","ref":"JSV.Validator.html#context/3"},{"type":"type","title":"JSV.Validator.eval_path_segment/0","doc":"","ref":"JSV.Validator.html#t:eval_path_segment/0"},{"type":"function","title":"JSV.Validator.flat_errors/1","doc":"","ref":"JSV.Validator.html#flat_errors/1"},{"type":"function","title":"JSV.Validator.list_evaluaded/1","doc":"","ref":"JSV.Validator.html#list_evaluaded/1"},{"type":"function","title":"JSV.Validator.merge_evaluated/2","doc":"","ref":"JSV.Validator.html#merge_evaluated/2"},{"type":"type","title":"JSV.Validator.path_segment/0","doc":"","ref":"JSV.Validator.html#t:path_segment/0"},{"type":"function","title":"JSV.Validator.reduce/4","doc":"Reduce over an enum with two accumulators, a user one, and the context.\n\n* The callback is called with `item, acc, vctx` for all items in the enum,\n  regardless of previously returned values. Returning and error tuple does not\n  stop the iteration.\n* When returning `{:ok, value, vctx}`, `value` will be the new user\n  accumulator, and the new context is carried on.\n* When returning `{:error, vctx}`, the current accumulator is not changed, but\n  the new returned context with errors is still carried on.\n* Returning an ok tuple after an error tuple on a previous item does not\n  remove the errors from the context struct.\n\nThe final return value is `{:ok, acc, vctx}` if all calls of the callback\nreturned an OK tuple, `{:error, vctx}` otherwise.\n\nThis is useful to call all possible validators for a given piece of data,\ncollecting all possible errors without stopping, but still returning an error\nin the end if some error arose.","ref":"JSV.Validator.html#reduce/4"},{"type":"type","title":"JSV.Validator.result/0","doc":"","ref":"JSV.Validator.html#t:result/0"},{"type":"function","title":"JSV.Validator.return/2","doc":"","ref":"JSV.Validator.html#return/2"},{"type":"function","title":"JSV.Validator.to_error/1","doc":"","ref":"JSV.Validator.html#to_error/1"},{"type":"function","title":"JSV.Validator.validate/3","doc":"Validate the given data with the given validator. The validator is typically a\nsub-part of a `JSV.Root` struct built with `JSV.build/2` such as a\n`JSV.Subschema` struct.","ref":"JSV.Validator.html#validate/3"},{"type":"function","title":"JSV.Validator.validate_as/4","doc":"Validates data with a sub part of the schema, for instance `if`, `then` or\n`else`. Data path will not change in the context.\n\nSee `validate_in/5` to validate sub terms of the data.","ref":"JSV.Validator.html#validate_as/4"},{"type":"function","title":"JSV.Validator.validate_detach/4","doc":"Validate the data with the given validators but separate the current\nevaluation context during the validation.\n\nCurrently evaluated properties or items will not be seen as evaluated during\nthe validation by the given `subschema`.","ref":"JSV.Validator.html#validate_detach/4"},{"type":"function","title":"JSV.Validator.validate_in/5","doc":"Validates a sub term of the data, identified by `key`, which can be a property\nname (a string), or an array index (an integer).\n\nSee `validate_as/4` to validate the same data point with a nested keyword. For\ninstance `if`, `then` or `else`.","ref":"JSV.Validator.html#validate_in/5"},{"type":"function","title":"JSV.Validator.validate_ref/4","doc":"","ref":"JSV.Validator.html#validate_ref/4"},{"type":"type","title":"JSV.Validator.validator/0","doc":"","ref":"JSV.Validator.html#t:validator/0"},{"type":"macro","title":"JSV.Validator.with_error/4","doc":"","ref":"JSV.Validator.html#with_error/4"},{"type":"module","title":"JSV.Validator.Error","doc":"Representation of an error encountered during validation.","ref":"JSV.Validator.Error.html"},{"type":"opaque","title":"JSV.Validator.Error.t/0","doc":"","ref":"JSV.Validator.Error.html#t:t/0"},{"type":"module","title":"JSV.Validator.ValidationContext","doc":"Validation context carried along by the `JSV.Validator` and given to all\nvocabulary implementations.\n\nThis struct is used to store errors found during validation, and to hold\ncontextual information such as the current path in the data or in the\nschema.","ref":"JSV.Validator.ValidationContext.html"},{"type":"type","title":"JSV.Validator.ValidationContext.t/0","doc":"","ref":"JSV.Validator.ValidationContext.html#t:t/0"},{"type":"behaviour","title":"JSV.FormatValidator","doc":"Behaviour for format validator implementations.\n\nSuch implementations must be given to `JSV.build/2` in the `:formats` option:\n\n    JSV.build!(raw_schema,\n      resolver: resolver,\n      formats: [MyModule | JSV.default_format_validator_modules()]\n    )\n\nEach given module is interrogated for format support when a schema is built.\nModules earlier in the list take precedence and if a format is found in the\nreturned value of the `c:supported_formats/0` callback, the module is selected\nfor compilation and no other module will be tried.\n\nA module can declare multiple formats.","ref":"JSV.FormatValidator.html"},{"type":"type","title":"JSV.FormatValidator.format/0","doc":"","ref":"JSV.FormatValidator.html#t:format/0"},{"type":"callback","title":"JSV.FormatValidator.supported_formats/0","doc":"Returns the list of the supported formats, as strings.","ref":"JSV.FormatValidator.html#c:supported_formats/0"},{"type":"callback","title":"JSV.FormatValidator.validate_cast/2","doc":"Receives the schema format as string, and the data as a string.\n\nReturns a result tuple with data optionally casted to a more meaningful data\nstructure (for instance returning a `Date` struct instead of the string\nrepresentation of the date).","ref":"JSV.FormatValidator.html#c:validate_cast/2"},{"type":"module","title":"JSV.FormatValidator.Default","doc":"Default implementation of format validation.\n\nRefer to the \"Formats\" section in the `JSV` documentation to know which\nformats are supported. This can depend on the current Elixir version and\navailable optional libraries.","ref":"JSV.FormatValidator.Default.html"},{"type":"behaviour","title":"JSV.Vocabulary","doc":"Behaviour for vocabulary implementation.\n\nA vocabulary module is used twice during the lifetime of a JSON schema:\n\n* When building a schema, the vocabulary module is given key/value pairs such\n  as `{\"type\", \"integer\"}` or `{\"properties\", map_of_schemas}` and must\n  consume or ignore the given keyword, storing custom validation data in an\n  accumulator for further use.\n* When validating a schema, the module is called with the data to validate and\n  the accumulated validation data to produce a validation result.","ref":"JSV.Vocabulary.html"},{"type":"macro","title":"JSV.Vocabulary.__using__/1","doc":"By using this module you will:\n\n* Declare that module as a behaviour\n* Import all macros from the module\n* Declare a `priority/0` function if the `:priority` option is provided.","ref":"JSV.Vocabulary.html#__using__/1"},{"type":"type","title":"JSV.Vocabulary.acc/0","doc":"Represents the accumulator initially returned by `c:init_validators/1` and\naccepted and returned by `c:handle_keyword/4`.\n\nThis accumulator is then given to `c:finalize_validators/1` and the\n`t:collection/0` type is used from there.","ref":"JSV.Vocabulary.html#t:acc/0"},{"type":"type","title":"JSV.Vocabulary.collection/0","doc":"Represents the final form of the collected keywords after the ultimate\ntransformation returned by `c:finalize_validators/1`.","ref":"JSV.Vocabulary.html#t:collection/0"},{"type":"macro","title":"JSV.Vocabulary.consume_keyword/1","doc":"Defines a `c:handle_keyword/4` callback that will return the current\naccumulator without changes, but preventing other vocabulary modules with\nlower priority (higher number) to be called with this keyword.\n\nThe keyword must be given in atom form.","ref":"JSV.Vocabulary.html#consume_keyword/1"},{"type":"type","title":"JSV.Vocabulary.data/0","doc":"","ref":"JSV.Vocabulary.html#t:data/0"},{"type":"callback","title":"JSV.Vocabulary.finalize_validators/1","doc":"","ref":"JSV.Vocabulary.html#c:finalize_validators/1"},{"type":"callback","title":"JSV.Vocabulary.format_error/3","doc":"","ref":"JSV.Vocabulary.html#c:format_error/3"},{"type":"callback","title":"JSV.Vocabulary.handle_keyword/4","doc":"","ref":"JSV.Vocabulary.html#c:handle_keyword/4"},{"type":"macro","title":"JSV.Vocabulary.ignore_any_keyword/0","doc":"Defines a `c:handle_keyword/4` callback that will return `:ignore` for any\ngiven value.\n\nGenerally used below `take_keyword/6`:\n\n      take_keyword :items, items when is_map(items), acc, builder, raw_schema do\n        # ...\n      end\n\n      ignore_any_keyword()","ref":"JSV.Vocabulary.html#ignore_any_keyword/0"},{"type":"macro","title":"JSV.Vocabulary.ignore_keyword/1","doc":"Defines a `c:handle_keyword/4` callback that will return `:ignore` for the\ngiven keyword. The keyword must be given in atom form.\n\nGenerally used below `take_keyword/6`:\n\n      take_keyword :items, items when is_map(items), acc, builder, raw_schema do\n        # ...\n      end\n\n      ignore_keyword(:additionalItems)\n      ignore_keyword(:prefixItems)","ref":"JSV.Vocabulary.html#ignore_keyword/1"},{"type":"callback","title":"JSV.Vocabulary.init_validators/1","doc":"","ref":"JSV.Vocabulary.html#c:init_validators/1"},{"type":"type","title":"JSV.Vocabulary.pair/0","doc":"","ref":"JSV.Vocabulary.html#t:pair/0"},{"type":"callback","title":"JSV.Vocabulary.priority/0","doc":"Returns the priority for applyting this module to the data.\n\nLower values (close to zero) will be applied first. You can think \"order\"\ninstead of \"priority\" but several modules can share the same priority value.\n\nThis can be useful to define vocabularies that depend on other vocabularies.\nFor instance, the `unevaluatedProperties` keyword needs \"properties\",\n\"patternProperties\", \"additionalProperties\" and \"allOf\", \"oneOf\", \"anyOf\",\n_etc._ to be ran before itself so it can lookup what has been evaluated.\n\nModules shipped in this library have priority of 100, 200, etc. up to 900 so\nyou can interleave your own vocabularies. Casting values to non-validable\nterms (such as structs or dates) should be done by vocabularies with a\npriority of 1000 and above.","ref":"JSV.Vocabulary.html#c:priority/0"},{"type":"function","title":"JSV.Vocabulary.take_integer/4","doc":"Adds the given integer to the list accumulator as a 2-tuple with the given\n`key`.\n\nFails if the value is not an integer. Floats with zero-fractional (as `123.0`)\nwill be accepted and converted to integer, as the JSON Schema spec dictates.","ref":"JSV.Vocabulary.html#take_integer/4"},{"type":"macro","title":"JSV.Vocabulary.take_keyword/6","doc":"An utility macro to ease declare vocabularies with atom keys.\n\nDefines the `c:handle_keyword/4` callback.\n\n**Important**\n\n- The keyword must be given in atom form.\n- The original goal was to allow atom keys and values everywhere. Schemas are\n  now converted to binary from before being built.\n- It is still useful to use this macro to the signature of the\n  `c:handle_keyword/4` callback can be changed easily without too much\n  refactoring.\n- Guards must be placed after the second argument:\n\n      take_keyword :items, items when is_map(items), acc, builder, raw_schema do\n        # ...\n      end","ref":"JSV.Vocabulary.html#take_keyword/6"},{"type":"function","title":"JSV.Vocabulary.take_number/4","doc":"Adds the given integer to the list accumulator as a 2-tuple with the given\n`key`.\n\nFails if the value is not a number.","ref":"JSV.Vocabulary.html#take_number/4"},{"type":"function","title":"JSV.Vocabulary.take_sub/4","doc":"Gives the sub raw schema to the builder and adds the build result in the list\naccumulator as a 2-tuple with the given `key`.","ref":"JSV.Vocabulary.html#take_sub/4"},{"type":"callback","title":"JSV.Vocabulary.validate/3","doc":"","ref":"JSV.Vocabulary.html#c:validate/3"},{"type":"module","title":"JSV.Vocabulary.Draft7.Applicator","doc":"Implementation of the applicator vocabulary with draft 7 sepecifiticies.","ref":"JSV.Vocabulary.Draft7.Applicator.html"},{"type":"module","title":"JSV.Vocabulary.Draft7.Content","doc":"Implementation of the content vocabulary with draft 7 sepecifiticies. No\nvalidation is performed.","ref":"JSV.Vocabulary.Draft7.Content.html"},{"type":"module","title":"JSV.Vocabulary.Draft7.Core","doc":"Implementation of the core vocabulary with draft 7 sepecifiticies.","ref":"JSV.Vocabulary.Draft7.Core.html"},{"type":"module","title":"JSV.Vocabulary.Draft7.Format","doc":"Implementation of the format vocabulary with draft 7 sepecifiticies.","ref":"JSV.Vocabulary.Draft7.Format.html"},{"type":"function","title":"JSV.Vocabulary.Draft7.Format.finalize_validators/1","doc":"","ref":"JSV.Vocabulary.Draft7.Format.html#finalize_validators/1"},{"type":"function","title":"JSV.Vocabulary.Draft7.Format.handle_keyword/4","doc":"","ref":"JSV.Vocabulary.Draft7.Format.html#handle_keyword/4"},{"type":"function","title":"JSV.Vocabulary.Draft7.Format.init_validators/1","doc":"","ref":"JSV.Vocabulary.Draft7.Format.html#init_validators/1"},{"type":"function","title":"JSV.Vocabulary.Draft7.Format.validate/3","doc":"","ref":"JSV.Vocabulary.Draft7.Format.html#validate/3"},{"type":"module","title":"JSV.Vocabulary.Draft7.MetaData","doc":"Implementation of the meta-data vocabulary with draft 7 sepecifiticies.","ref":"JSV.Vocabulary.Draft7.MetaData.html"},{"type":"module","title":"JSV.Vocabulary.Draft7.Validation","doc":"Implementation of the validation vocabulary with draft 7 sepecifiticies.","ref":"JSV.Vocabulary.Draft7.Validation.html"},{"type":"function","title":"JSV.Vocabulary.Draft7.Validation.finalize_validators/1","doc":"","ref":"JSV.Vocabulary.Draft7.Validation.html#finalize_validators/1"},{"type":"function","title":"JSV.Vocabulary.Draft7.Validation.handle_keyword/4","doc":"","ref":"JSV.Vocabulary.Draft7.Validation.html#handle_keyword/4"},{"type":"function","title":"JSV.Vocabulary.Draft7.Validation.init_validators/1","doc":"","ref":"JSV.Vocabulary.Draft7.Validation.html#init_validators/1"},{"type":"function","title":"JSV.Vocabulary.Draft7.Validation.validate/3","doc":"","ref":"JSV.Vocabulary.Draft7.Validation.html#validate/3"},{"type":"module","title":"JSV.Vocabulary.V202012.Applicator","doc":"Implementation for the `https://json-schema.org/draft/2020-12/vocab/applicator`\nvocabulary.","ref":"JSV.Vocabulary.V202012.Applicator.html"},{"type":"function","title":"JSV.Vocabulary.V202012.Applicator.validate_keyword/3","doc":"","ref":"JSV.Vocabulary.V202012.Applicator.html#validate_keyword/3"},{"type":"module","title":"JSV.Vocabulary.V202012.Content","doc":"Invactive implementation for the\n`https://json-schema.org/draft/2020-12/vocab/content` vocabulary. No\nvalidation is performed.","ref":"JSV.Vocabulary.V202012.Content.html"},{"type":"module","title":"JSV.Vocabulary.V202012.Core","doc":"Implementation for the `https://json-schema.org/draft/2020-12/vocab/core`\nvocabulary.","ref":"JSV.Vocabulary.V202012.Core.html"},{"type":"module","title":"JSV.Vocabulary.V202012.Format","doc":"Implementation for the\n`https://json-schema.org/draft/2020-12/vocab/format-annotation` and\n`https://json-schema.org/draft/2020-12/vocab/format-assertion` vocabularies.","ref":"JSV.Vocabulary.V202012.Format.html"},{"type":"module","title":"JSV.Vocabulary.V202012.MetaData","doc":"Implementation for the `https://json-schema.org/draft/2020-12/vocab/meta-data`\nvocabulary.","ref":"JSV.Vocabulary.V202012.MetaData.html"},{"type":"module","title":"JSV.Vocabulary.V202012.Unevaluated","doc":"Implementation for the\n`https://json-schema.org/draft/2020-12/vocab/unevaluated` vocabulary.","ref":"JSV.Vocabulary.V202012.Unevaluated.html"},{"type":"function","title":"JSV.Vocabulary.V202012.Unevaluated.validate_keyword/3","doc":"","ref":"JSV.Vocabulary.V202012.Unevaluated.html#validate_keyword/3"},{"type":"module","title":"JSV.Vocabulary.V202012.Validation","doc":"Implementation for the\n`https://json-schema.org/draft/2020-12/vocab/validation` vocabulary.","ref":"JSV.Vocabulary.V202012.Validation.html"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.36.1"}}